@@ -85,8 +85,7 @@ static int decrypt_data (schema_id, ASN1_TYPE pkcs8_asn, const char *root,
 			 const struct pbe_enc_params *enc_params,
 			 gnutls_datum_t * decrypted_data);
 static int decode_private_key_info (const gnutls_datum_t * der,
-				    gnutls_x509_privkey_t pkey,
-				    ASN1_TYPE * out);
+				    gnutls_x509_privkey_t pkey);
 static int write_schema_params (schema_id schema, ASN1_TYPE pkcs8_asn,
 				const char *where,
 				const struct pbkdf2_params *kdf_params,
@@ -127,6 +126,89 @@ check_schema (const char *oid)
   return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 }
 
+/* Encodes a private key to the raw format PKCS #8 needs.
+ * For RSA it is a PKCS #1 DER private key and for DSA it is
+ * an ASN.1 INTEGER of the x value.
+ */
+inline static int
+_encode_privkey (gnutls_x509_privkey pkey, gnutls_datum * raw)
+{
+  size_t size = 0;
+  opaque *data = NULL;
+  int ret;
+  ASN1_TYPE spk = ASN1_TYPE_EMPTY;
+
+  switch (pkey->pk_algorithm)
+    {
+    case GNUTLS_PK_RSA:
+      ret =
+	gnutls_x509_privkey_export (pkey, GNUTLS_X509_FMT_DER, NULL, &size);
+      if (ret != GNUTLS_E_SHORT_MEMORY_BUFFER)
+	{
+	  gnutls_assert ();
+	  goto error;
+	}
+
+      data = gnutls_malloc (size);
+      if (data == NULL)
+	{
+	  gnutls_assert ();
+	  ret = GNUTLS_E_MEMORY_ERROR;
+	  goto error;
+	}
+
+
+      ret =
+	gnutls_x509_privkey_export (pkey, GNUTLS_X509_FMT_DER, data, &size);
+      if (ret < 0)
+	{
+	  gnutls_assert ();
+	  goto error;
+	}
+
+      raw->data = data;
+      raw->size = size;
+      break;
+    case GNUTLS_PK_DSA:
+      /* DSAPublicKey == INTEGER */
+      if ((ret = asn1_create_element
+	   (_gnutls_get_gnutls_asn (), "GNUTLS.DSAPublicKey", &spk))
+	  != ASN1_SUCCESS)
+	{
+	  gnutls_assert ();
+	  return _gnutls_asn2err (ret);
+	}
+
+      ret = _gnutls_x509_write_int (spk, "", pkey->params[4], 0);
+      if (ret < 0)
+	{
+	  gnutls_assert ();
+	  goto error;
+	}
+      ret = _gnutls_x509_der_encode (spk, "", raw, 0);
+      if (ret < 0)
+	{
+	  gnutls_assert ();
+	  goto error;
+	}
+
+      asn1_delete_structure (&spk);
+      break;
+
+    default:
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  return 0;
+  
+  error:
+    gnutls_free(data);
+    asn1_delete_structure (&spk);
+    return ret;
+
+}
+
 /* 
  * Encodes a PKCS #1 private key to a PKCS #8 private key
  * info. The output will be allocated and stored into der. Also
@@ -137,16 +219,37 @@ encode_to_private_key_info (gnutls_x509_privkey_t pkey,
 			    gnutls_datum_t * der, ASN1_TYPE * pkey_info)
 {
   int result, len;
-  size_t size;
-  opaque *data = NULL;
   opaque null = 0;
+  const char *oid;
+  gnutls_datum algo_params = { NULL, 0 };
+  gnutls_datum algo_privkey = { NULL, 0 };
 
-  if (pkey->pk_algorithm != GNUTLS_PK_RSA)
+  if (pkey->pk_algorithm != GNUTLS_PK_RSA
+      && pkey->pk_algorithm != GNUTLS_PK_DSA)
     {
       gnutls_assert ();
       return GNUTLS_E_UNIMPLEMENTED_FEATURE;
     }
 
+  if (pkey->pk_algorithm == GNUTLS_PK_RSA)
+    {
+      oid = PK_PKIX1_RSA_OID;
+      /* parameters are null 
+       */
+    }
+  else
+    {
+      oid = PK_DSA_OID;
+      result =
+	_gnutls_x509_write_dsa_params (pkey->params, pkey->params_size,
+				       &algo_params);
+      if (result < 0)
+	{
+	  gnutls_assert ();
+	  return result;
+	}
+    }
+
   if ((result =
        asn1_create_element (_gnutls_get_pkix (),
 			    "PKIX1.pkcs-8-PrivateKeyInfo",
@@ -171,8 +274,7 @@ encode_to_private_key_info (gnutls_x509_privkey_t pkey,
    * fields. (OID+NULL data)
    */
   result =
-    asn1_write_value (*pkey_info, "privateKeyAlgorithm.algorithm",
-		      PK_PKIX1_RSA_OID, 1);
+    asn1_write_value (*pkey_info, "privateKeyAlgorithm.algorithm", oid, 1);
   if (result != ASN1_SUCCESS)
     {
       gnutls_assert ();
@@ -181,7 +283,9 @@ encode_to_private_key_info (gnutls_x509_privkey_t pkey,
     }
 
   result =
-    asn1_write_value (*pkey_info, "privateKeyAlgorithm.parameters", NULL, 0);
+    asn1_write_value (*pkey_info, "privateKeyAlgorithm.parameters",
+		      algo_params.data, algo_params.size);
+  _gnutls_free_datum (&algo_params);
   if (result != ASN1_SUCCESS)
     {
       gnutls_assert ();
@@ -189,38 +293,20 @@ encode_to_private_key_info (gnutls_x509_privkey_t pkey,
       goto error;
     }
 
+
   /* Write the raw private key
    */
-  size = 0;
-  result =
-    gnutls_x509_privkey_export (pkey, GNUTLS_X509_FMT_DER, NULL, &size);
-  if (result != GNUTLS_E_SHORT_MEMORY_BUFFER)
-    {
-      gnutls_assert ();
-      goto error;
-    }
-
-  data = gnutls_alloca (size);
-  if (data == NULL)
-    {
-      gnutls_assert ();
-      result = GNUTLS_E_MEMORY_ERROR;
-      goto error;
-    }
-
-
-  result =
-    gnutls_x509_privkey_export (pkey, GNUTLS_X509_FMT_DER, data, &size);
+  result = _encode_privkey (pkey, &algo_privkey);
   if (result < 0)
     {
       gnutls_assert ();
       goto error;
     }
 
-  result = asn1_write_value (*pkey_info, "privateKey", data, size);
-
-  gnutls_afree (data);
-  data = NULL;
+  result =
+    asn1_write_value (*pkey_info, "privateKey", algo_privkey.data,
+		      algo_privkey.size);
+  _gnutls_free_datum (&algo_privkey);
 
   if (result != ASN1_SUCCESS)
     {
@@ -272,10 +358,8 @@ encode_to_private_key_info (gnutls_x509_privkey_t pkey,
 
 error:
   asn1_delete_structure (pkey_info);
-  if (data != NULL)
-    {
-      gnutls_afree (data);
-    }
+  _gnutls_free_datum (&algo_params);
+  _gnutls_free_datum (&algo_privkey);
   return result;
 
 }
@@ -405,10 +489,9 @@ error:
   *   replaced by the actual size of parameters)
   *
   * This function will export the private key to a PKCS8 structure.
-  * Currently only RSA keys can be exported since there is no documented
-  * standard for other keys. If the flags do not
-  * specify the encryption cipher, then the default 3DES (PBES2) will
-  * be used.
+  * Currently RSA and DSA keys can be exported. For DSA keys we use
+  * PKCS #11 definitions. If the flags do not specify the encryption 
+  * cipher, then the default 3DES (PBES2) will be used.
   *
   * The @password can be either ASCII or UTF-8 in the default PBES2
   * encryption schemas, or ASCII for the PKCS12 schemas.
@@ -471,7 +554,7 @@ gnutls_x509_privkey_export_pkcs8 (gnutls_x509_privkey_t key,
       ret =
 	_gnutls_x509_export_int (pkey_info, format,
 				 PEM_UNENCRYPTED_PKCS8,
-				 *output_data_size, output_data,
+				 output_data,
 				 output_data_size);
 
       asn1_delete_structure (&pkey_info);
@@ -491,7 +574,7 @@ gnutls_x509_privkey_export_pkcs8 (gnutls_x509_privkey_t key,
 
       ret =
 	_gnutls_x509_export_int (pkcs8_asn, format, PEM_PKCS8,
-				 *output_data_size, output_data,
+				 output_data,
 				 output_data_size);
 
       asn1_delete_structure (&pkcs8_asn);
@@ -649,13 +732,12 @@ error:
 static int
 decode_pkcs8_key (const gnutls_datum_t * raw_key,
 		  const char *password,
-		  gnutls_x509_privkey_t pkey, ASN1_TYPE * out)
+		  gnutls_x509_privkey_t pkey)
 {
   int result, len;
   char enc_oid[64];
   gnutls_datum_t tmp;
   ASN1_TYPE pbes2_asn = ASN1_TYPE_EMPTY, pkcs8_asn = ASN1_TYPE_EMPTY;
-  ASN1_TYPE ret_asn;
   int params_start, params_end, params_len;
   struct pbkdf2_params kdf_params;
   struct pbe_enc_params enc_params;
@@ -734,7 +816,7 @@ decode_pkcs8_key (const gnutls_datum_t * raw_key,
 
   asn1_delete_structure (&pkcs8_asn);
 
-  result = decode_private_key_info (&tmp, pkey, &ret_asn);
+  result = decode_private_key_info (&tmp, pkey);
   _gnutls_free_datum (&tmp);
 
   if (result < 0)
@@ -762,8 +844,6 @@ decode_pkcs8_key (const gnutls_datum_t * raw_key,
       goto error;
     }
 
-  *out = ret_asn;
-
   return 0;
 
 error:
@@ -772,16 +852,111 @@ error:
   return result;
 }
 
+/* Decodes an RSA privateKey from a PKCS8 structure.
+ */
+static int _decode_pkcs8_rsa_key( ASN1_TYPE pkcs8_asn, gnutls_x509_privkey pkey)
+{
+int ret;
+gnutls_datum tmp;
+
+  ret = _gnutls_x509_read_value(pkcs8_asn, "privateKey", &tmp, 0);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto error;
+    }
+
+  pkey->key = _gnutls_privkey_decode_pkcs1_rsa_key (&tmp, pkey);
+  _gnutls_free_datum( &tmp);
+  if (pkey->key == NULL)
+    {
+      gnutls_assert ();
+      goto error;
+    }
+
+  return 0;
+
+error:
+  gnutls_x509_privkey_deinit(pkey);
+  return ret;
+}
+
+/* Decodes an DSA privateKey and params from a PKCS8 structure.
+ */
+static int _decode_pkcs8_dsa_key( ASN1_TYPE pkcs8_asn, gnutls_x509_privkey pkey)
+{
+int ret;
+gnutls_datum tmp;
+mpi_t y;
+
+  ret = _gnutls_x509_read_value(pkcs8_asn, "privateKey", &tmp, 0);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto error;
+    }
+
+  ret = _gnutls_x509_read_der_int( tmp.data, tmp.size, &pkey->params[4]);
+  _gnutls_free_datum( &tmp);
+  
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto error;
+    }
+
+  ret = _gnutls_x509_read_value( pkcs8_asn, "privateKeyAlgorithm.parameters", &tmp, 0);
+  if (ret < 0) 
+    {
+      gnutls_assert();
+      goto error;
+    }
+  
+  ret = _gnutls_x509_read_dsa_params (tmp.data, tmp.size, pkey->params);
+  _gnutls_free_datum( &tmp);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      goto error;
+    }
+  
+  /* the public key can be generated as g^x mod p */
+  pkey->params[3] = _gnutls_mpi_alloc_like (pkey->params[0]);
+  if (pkey->params[3] == NULL)
+    {
+      gnutls_assert();
+      goto error;
+    }
+  
+  _gnutls_mpi_powm (pkey->params[3], pkey->params[2], pkey->params[4], pkey->params[0]);
+
+  if (!pkey->crippled)
+    {
+      ret = _gnutls_asn1_encode_dsa (&pkey->key, pkey->params);
+      if (ret < 0)
+	{
+	  gnutls_assert ();
+	  goto error;
+	}
+    }
+
+  pkey->params_size = DSA_PRIVATE_PARAMS;
+
+  return 0;
+
+error:
+  gnutls_x509_privkey_deinit(pkey);
+  return ret;
+}
+
+
 static int
 decode_private_key_info (const gnutls_datum_t * der,
-			 gnutls_x509_privkey_t pkey, ASN1_TYPE * out)
+			 gnutls_x509_privkey_t pkey)
 {
   int result, len;
-  opaque oid[64], *data = NULL;
-  gnutls_datum_t tmp;
+  opaque oid[64];
   ASN1_TYPE pkcs8_asn = ASN1_TYPE_EMPTY;
-  ASN1_TYPE ret_asn;
-  int data_size;
 
 
   if ((result =
@@ -814,9 +989,13 @@ decode_private_key_info (const gnutls_datum_t * der,
       goto error;
     }
 
-  /* we only support RSA private keys.
+  /* we only support RSA and DSA private keys.
    */
-  if (strcmp (oid, PK_PKIX1_RSA_OID) != 0)
+  if (strcmp (oid, PK_PKIX1_RSA_OID) == 0)
+    pkey->pk_algorithm = GNUTLS_PK_RSA;
+  else if (strcmp (oid, PK_DSA_OID) == 0)
+    pkey->pk_algorithm = GNUTLS_PK_DSA;
+  else
     {
       gnutls_assert ();
       _gnutls_x509_log
@@ -827,54 +1006,23 @@ decode_private_key_info (const gnutls_datum_t * der,
 
   /* Get the DER encoding of the actual private key.
    */
-  data_size = 0;
-  result = asn1_read_value (pkcs8_asn, "privateKey", NULL, &data_size);
-  if (result != ASN1_MEM_ERROR)
+
+  if (pkey->pk_algorithm == GNUTLS_PK_RSA)
+    result = _decode_pkcs8_rsa_key( pkcs8_asn, pkey);
+  else if (pkey->pk_algorithm == GNUTLS_PK_DSA)
+    result = _decode_pkcs8_dsa_key( pkcs8_asn, pkey);
+  
+  if (result < 0)
     {
-      gnutls_assert ();
-      result = _gnutls_asn2err (result);
-      goto error;
+      gnutls_assert();
+      return result;
     }
 
-  data = gnutls_alloca (data_size);
-  if (data == NULL)
-    {
-      gnutls_assert ();
-      result = GNUTLS_E_MEMORY_ERROR;
-      goto error;
-    }
-
-  result = asn1_read_value (pkcs8_asn, "privateKey", data, &data_size);
-  if (result != ASN1_SUCCESS)
-    {
-      gnutls_assert ();
-      result = _gnutls_asn2err (result);
-      goto error;
-    }
-
-  asn1_delete_structure (&pkcs8_asn);
-
-  tmp.data = data;
-  tmp.size = data_size;
-
-  pkey->pk_algorithm = GNUTLS_PK_RSA;
-
-  ret_asn = _gnutls_privkey_decode_pkcs1_rsa_key (&tmp, pkey);
-  if (ret_asn == NULL)
-    {
-      gnutls_assert ();
-    }
-
-  *out = ret_asn;
-
-  return 0;
+  result = 0;
 
 error:
   asn1_delete_structure (&pkcs8_asn);
-  if (data != NULL)
-    {
-      gnutls_afree (data);
-    }
+
   return result;
 
 }
@@ -890,7 +1038,7 @@ error:
   *
   * This function will convert the given DER or PEM encoded PKCS8 2.0 encrypted key
   * to the native gnutls_x509_privkey_t format. The output will be stored in @key.
-  * Currently only RSA keys can be imported, and flags can only be used to indicate
+  * Currently RSA and DSA keys can be imported, and flags can only be used to indicate
   * an unencrypted key.
   *
   * The @password can be either ASCII or UTF-8 in the default PBES2
@@ -960,11 +1108,11 @@ gnutls_x509_privkey_import_pkcs8 (gnutls_x509_privkey_t key,
 
   if (flags & GNUTLS_PKCS_PLAIN)
     {
-      result = decode_private_key_info (&_data, key, &key->key);
+      result = decode_private_key_info (&_data, key);
     }
   else
     {				/* encrypted. */
-      result = decode_pkcs8_key (&_data, password, key, &key->key);
+      result = decode_pkcs8_key (&_data, password, key);
     }
 
   if (result < 0)
