@@ -365,6 +365,7 @@ gnutls_session_t
 initialize_session (void)
 {
   gnutls_session_t session;
+  const char *err;
 
   gnutls_init (&session, GNUTLS_SERVER);
 
@@ -380,7 +381,11 @@ initialize_session (void)
       gnutls_db_set_ptr (session, NULL);
     }
 
-  gnutls_set_default_priority2 (session, GNUTLS_PRIORITIES_PERFORMANCE);
+  if (gnutls_priority_set_direct (session, info.priorities, &err) < 0)
+    {
+      fprintf(stderr, "Syntax error at: %s\n", err);
+      exit(1);
+    }
 
   if (cipher_priority[0])
     gnutls_cipher_set_priority (session, cipher_priority);
