@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006  Free Software Foundation, Inc.
+ * Copyright (C) 2006, 2007  Free Software Foundation, Inc.
  *
  * Author: Simon Josefsson
  *
@@ -22,6 +22,10 @@
  *
  */
 
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
